part of tecfy_corev2_client_package;

class EntityListController extends BaseController {
  late Entity entity;
  bool isPopup;

  /// data
  List<dynamic>? data;
  List<dynamic>? filterdData;
  List<EntityFilter>? filters;

  late List<Property> searchableProperties;

  int pageIndex = 0;
  int? totalCount;
  int pageIndexMax = 0;
  bool hasNextPage = true;
  late ScrollController scrollController;
  late TextEditingController searchController;
  late RefreshController refreshController =
      RefreshController(initialRefresh: false); // for pull to refrsh
  IEntityAction? entityActionsService;
  EntityListController(State<StatefulWidget> state, String entityName,
      {this.filters,
      this.orderBy,
      this.orderByDesc = false,
      required this.isPopup})
      : super(state) {
    scrollController = ScrollController();
    refreshController.footerMode?.addListener(footerListner);
    searchController = TextEditingController();
    filters ??= [];
    if (kDebugMode) print('-------------- EntityListController => $entityName');
    entity = EntityService.getEntity(entityName) ?? Entity.fromMap({});
    searchableProperties =
        entity.properties.where((e) => e.isSearchable).toList();

    entityActionsService =
        TecfyCoreApp.entityActionsServices[entity.dbName.toLowerCase()];

    // _loadMainFilter();
    loadData();
  }

  @override
  dispose() {
    try {
      refreshController.dispose();
    } catch (err) {}
    try {
      searchController.dispose();
    } catch (err) {}
    try {
      scrollController.dispose();
    } catch (err) {}
    super.dispose();
  }

  String? orderBy;
  bool orderByDesc = true;

  addChildren(items, List<dynamic> rows) {
    for (var item in items) {
      rows.add(item);
      if (item['_expanded'] == true &&
          item['_children'] != null &&
          item['_children'].length > 0) {
        rows = addChildren(item['_children'], rows);
      }
    }
    return rows;
  }

  Future<void> onOpenFilter() async {
    if (kDebugMode) print('-------------------- before dialog result $filters');
    List<Property> properties = entity.propertiesRow
        .map<Property>((p) => Property.getProperty(p)!)
        .toList();

    List<Property> propsWithoutFilter = properties.map((prop) {
      if (prop is PropertyDropdown) {
        prop.isFilter = false;
      }
      if (prop is PropertyMultiDropdown) {
        prop.isFilter = false;
      }
      if (prop is PropertyManyToMany) {
        prop.isFilter = false;
      }
      return prop;
    }).toList();

    var filterResult = await AppNavigator.showAppDialog(
      context,
      (c) => EntityFilterPage(propsWithoutFilter, filters),
    );

    if (kDebugMode)
      print(
          '-------------------- after dialog result $filterResult   ====== $filters');
    if (filterResult != null && filterResult is List<EntityFilter>) {
      filterResult = filterResult
          .where((e) =>
              e.value != null && (e.propertyName != null || e.property != null))
          .toList();
      if (filterResult.length != 0) {
        filters = filterResult;
        loadData();
      }
    }
  }

  Future<void> onOpenMerge() async {
    var mergeResult = await AppNavigator.showAppDialog(
        context, (c) => EntityMergePage(entity: entity));
    if (mergeResult == true) {
      loadData();
    }
  }

  Future<void> onExport() async {
    var header = ApiService().baseHeader
      ..addAll({'Authorization': 'Bearer ${ApiService.token}'});
    AppNavigator.showLoading(context);
    var response = await http.get(
        Uri.parse(
            '${ApiService.apiURL}admin/setup/export/${entity.dbName}?filter=${json.encode(filters?.map((e) => e.toJson()).toList())}&orderBy=${orderBy ?? ''}&orderByDesc=$orderByDesc&tex'
            'tSearch=${searchController.text}'),
        headers: header);
    AppNavigator.resume();
    if (response.statusCode == 400) {
      AppNavigator.showMessage(context, response.body, MessageType.error);
    } else {
      final bodyBytes = Uint8List.fromList(response.bodyBytes);
      await Printing.sharePdf(
          bytes: bodyBytes, filename: '${entity.name}-لا يمكن ادراجه.xlsx');
    }
  }

  Future globalSearchText({bool clearPagination = false}) async {
    if (ConnectionService.isOfflineMode && searchController.text.isNotEmpty) {
      globalSearchTextServer(clearPagination: clearPagination);
      return;
    }
    if (searchController.text.isEmpty) {
      filterdData = data;
    } else {
      filterdData = data?.where((d) {
        return searchableProperties.any((p) {
          print(d[p.dbName]);
          return d[p.dbName].toString().contains(searchController.text);
        });
      }).toList();
    }

    if (entity.isTree) {
      filterdData = addChildren(filterdData, List.empty(growable: true));
    }
    setState();
  }

  Future globalSearchTextServer({bool clearPagination = false}) async {
    if (searchController.text.isEmpty) {
      //filterdData = data;
      await loadFromServer();
      return;
    }
    if (kDebugMode) print('start search server -> ${searchController.text}');
    var result;
    // case: search from server in page after index zero
    if (clearPagination) {
      result = await EntityService.getDataSearch(
        entity,
        searchController.text,
        pageIndex: 0,
      );
      if (kDebugMode) print(result);
      if (!result.success) {
        result.showMessage(context);
      } else {
        pageIndex = 0;
        pageIndexMax = 0;
      }

      if (result.data != null) {
        if (kDebugMode)
          print('-------------- ${result.data?.length}  records loaded ');
        filterdData = result.data;
      } else {
        filterdData = [];
      }
      if (entity.isTree) {
        filterdData = addChildren(filterdData, List.empty(growable: true));
      }
    } else {
      // case: search from server in page index zero
      result = await EntityService.getDataSearch(
        entity,
        searchController.text,
        pageIndex: pageIndex,
      );
      if (kDebugMode) print(result);
      if (!result.success) {
        result.showMessage(context);
      }

      if (result.data != null) {
        if (kDebugMode)
          print('-------------- ${result.data?.length}  records loaded ');
        filterdData = result.data;
      } else {
        filterdData = [];
      }
      if (entity.isTree) {
        filterdData = addChildren(filterdData, List.empty(growable: true));
      }
    }

    setState();
  }

  sortBy(Property prop) async {
    if (kDebugMode) print('sort by ${prop.dbName}');
    if (prop.dbName != orderBy) {
      orderBy = prop.dbName;
      orderByDesc = false;
    } else {
      orderByDesc = !orderByDesc;
    }
    setLoading();
    await globalSearchTextServer();
    setResume();
  }

  bool _hasPageing = false;
  bool get hasPageing {
    if ((data?.length ?? 0) >= EntityService.pageSize) _hasPageing = true;
    return _hasPageing;
  }

  footerListner() async {
    if (kDebugMode)
      print('==========on load ====== ${refreshController.footerStatus}');
    if (refreshController.footerStatus != LoadStatus.loading) return;
    var success = await gotoNextPageIndex();
    if (success && hasNextPage) {
      refreshController.loadComplete();
    } else if (success && !hasNextPage) {
      //refreshController.loadComplete();
      refreshController.loadNoData();
    } else {
      refreshController.loadFailed();
    }
  }

  Future<bool> gotoNextPageIndex() {
    pageIndex++;
    pageIndexMax = max(pageIndexMax, pageIndex);
    return loadData(withLoading: false);
  }

  gotoPageIndex(int i) {
    pageIndex = i;
    setLoading();
    loadData();
  }

  Future getTotalCount() async {
    setLoading();
    if (searchController.text.isEmpty) {
      var result = await EntityService.getCount(entity,
          filter: filters ?? [], filterParentId: entity.isTree);
      if (result.data != null) {
        totalCount = result.data;
      }
    } else {
      List<EntityFilter> f = [];
      for (var prop in entity.properties.where((p) => p.isSearchable)) {
        f.add(EntityFilter(
            propertyName: prop.dbName,
            operatorId: prop.valueType == ValueTypes.Number ? 'E' : 'C',
            value: searchController.text));
      }
      var d = (await EntityService.getCount(entity,
              filter: f, filterOperator: 'or'))
          .data;
      if (d != null) totalCount = d;
    }
    setResume();
    setState();
  }

  var isLoading = false;
  Future<bool> loadData({bool withLoading = true}) async {
    if (isLoading) return true;
    isLoading = true;
    if (searchController.text.isEmpty) {
      //normal case without search text
      var result = await EntityService.getData(entity,
          orderBy: orderBy,
          orderByDesc: orderByDesc,
          filter: filters,
          pageIndex: pageIndex,
          filterParentId: entity.isTree);

      if (withLoading) setResume(result.success);
      if (!result.success || result.data == null) {
        //data = [];
        isLoading = false;
        if (withLoading) {
          setState();
        }
        if (ConnectionService.isOnlineMode) {
          showMessage(
            'Error Loading From Server',
            MessageType.error,
            message: result.message,
          );
        } else {
          showMessage('Error Loading From Local Database', MessageType.error);
        }
        globalSearchText();
        return false;
      } else {
        if (LayoutService.isMobile(context) &&
            pageIndex > 0 &&
            !entity.isLocal) {
          data?.addAll(result.data ?? []);
        } else {
          data = result.data;
        }
        if ((result.data?.length ?? 0) < EntityService.pageSize)
          hasNextPage = false;
      }
      globalSearchText();
      // if(pageIndex == 0) refreshAnimation();
      isLoading = false;
      return true;
    } else {
      //case with search text
      var result = await EntityService.getDataSearch(
        entity,
        searchController.text,
        pageIndex: pageIndex,
      );

      if (withLoading) setResume(result.success);
      if (!result.success || result.data == null) {
        //data = [];
        isLoading = false;
        if (withLoading) {
          setState();
        }
        if (ConnectionService.isOnlineMode) {
          showMessage(
            'Error Loading From Server',
            MessageType.error,
            message: result.message,
          );
        } else {
          showMessage('Error Loading From Local Database', MessageType.error);
        }
        globalSearchText();
        return false;
      } else {
        if (LayoutService.isMobile(context) &&
            pageIndex > 0 &&
            !entity.isLocal) {
          data?.addAll(result.data ?? []);
        } else {
          data = result.data;
        }
        if ((result.data?.length ?? 0) < EntityService.pageSize)
          hasNextPage = false;
      }
      // globalSearchText();
      filterdData = data;
      if (entity.isTree) {
        filterdData = addChildren(filterdData, List.empty(growable: true));
      }
      setState();
      // if(pageIndex == 0) refreshAnimation();
      isLoading = false;
      return true;
    }
  }

  Map<String, dynamic> getFiltersData() {
    Map<String, dynamic> dat = {};
    if (filters == null) return dat;
    for (var d in filters!) {
      if (d.propertyName != null) dat[d.propertyName!] = d.value;
    }
    return dat;
  }

  mainFilterChanged(Property prop, dynamic value) {
    filters ??= [];
    if (value == null) {
      filters?.removeWhere((element) =>
          element.property?.dbName == prop.dbName ||
          element.propertyName == prop.dbName);
    }

    var filter = filters!
        // ignore: unnecessary_cast
        .map<EntityFilter?>((e) => e as EntityFilter?)
        .firstWhere(
            (e) =>
                e!.propertyName == prop.dbName ||
                e.property?.dbName == prop.dbName,
            orElse: () => null);
    if (filter != null) {
      filter.value = value;
    } else {
      if (value != null)
        filters!.add(EntityFilter(
            property: prop,
            propertyName: prop.dbName,
            operatorId: 'E',
            value: value));
    }
    saveMainFilter();
    loadData(withLoading: true);
  }

  void clearMainFilter() {
    ConfigService.setValueString('MainFilter', null);
  }

  void saveMainFilter() {
    if (filters == null || entity.mainFilterPropertyDbNames.length == 0) return;
    var f = filters!
        .where((e) => entity.mainFilterPropertyDbNames.contains(e.propertyName))
        .toList();

    var oldFilter = getMainFilter()
        .where(
            (e) => !f.any((element) => element.propertyName == e.propertyName))
        .toList();

    oldFilter.addAll(f);
    ConfigService.setValueString(
        'MainFilter', jsonEncode(oldFilter.map((e) => e.toJson()).toList()));
  }

  List<EntityFilter> getMainFilter() {
    var filterString = ConfigService.getValueString('MainFilter');
    if (filterString.isEmpty) return [];
    return jsonDecode(filterString)
        .map<EntityFilter>((e) => EntityFilter.fromJson(e))
        .toList();
  }

  void _loadMainFilter() {
    filters = getMainFilter()
        .where((e) => entity.mainFilterPropertyDbNames.contains(e.propertyName))
        .toList();
  }

  Future loadFromServer() async {
    await loadData();

    refreshController.refreshCompleted();
  }

  Future onRefresh() async {
    pageIndex = 0;
    pageIndexMax = 0;
    hasNextPage = true;
    setLoading();
    await loadData();
  }

  Future onAdd() async {
    if (isPopup) {
      var result = await AppNavigator.showAppDialogEx(
          context, () => EntityFormPage(entity.dbName, isPopup: true),
          size: DialogBoxSizes.large);
      if (result == true) {
        loadFromServer();
      }
      return;
    }
    if (entityActionsService?.hasAddPage() ?? false) {
      await entityActionsService!.addPage(context);
      //if (result == true) {
      loadFromServer();
      // }
    } else {
      var result = await AppNavigator.navigateTo(
          context, () => EntityFormPage(entity.dbName),
          animationType: LayoutService.isMobile(context)
              ? AnimationTypes.slideUp
              : AnimationTypes.opacity);
      if (result == true) {
        loadFromServer();
      }
    }
  }

  Future onUpdate(d) async {
    if (isPopup) {
      var result = await AppNavigator.showAppDialogEx(
          context, () => EntityFormPage(entity.dbName, data: d, isPopup: true),
          size: DialogBoxSizes.large);
      if (result == true) {
        loadFromServer();
      }
      return;
    }
    if (entityActionsService?.hasEditPage(d) ?? false) {
      await entityActionsService!.editPage(context, d);
      loadFromServer();
    } else {
      var result = await AppNavigator.navigateTo(
          context, () => EntityFormPage(entity.dbName, data: d),
          animationType: LayoutService.isMobile(context)
              ? AnimationTypes.slideUp
              : AnimationTypes.opacity);
      if (result == true) {
        loadFromServer();
      }
    }
    setState();
  }

  Future onView(d) async {
    if (entityActionsService?.hasEditPage(d) ?? false) {
      await entityActionsService!.editPage(context, d);
      loadFromServer();
    } else {
      var result = await AppNavigator.navigateTo(
          context, () => EntityFormPage(entity.dbName, data: d),
          animationType: LayoutService.isMobile(context)
              ? AnimationTypes.slideUp
              : AnimationTypes.opacity);
      if (result == true) {
        loadFromServer();
      }
    }
  }

  Future onDelete(d) async {
    if (!entity.permission.canDelete ||
        entity.entityType == EntityTypes.ViewOnly ||
        entity.entityType == EntityTypes.Lookup) {
      return showMessage('لا يوجد صلاحيه للحذف', MessageType.error);
    }
    if (await confirmMessage('هل انت متاكد من الحذف؟')) {
      var result =
          await EntityService().delete(entity, d['_id'] ?? d['id'] ?? '');
      if (result.success) {
        showMessage('تم الحذف بنجاح', MessageType.success);
        data?.removeWhere((e) => e['_id'] == d['_id']);
        globalSearchText();
        //loadFromServer();
      } else {
        showMessage(
          'خطأ اثناء الحذف',
          MessageType.error,
          message: 'الرجاء التاكد من ان العنصر غير مرتبط بعناصر اخرى',
        );
      }
    }
  }

  Future expandItem(Map<String, dynamic> d) async {
    try {
      if (!entity.isTree) return true;
      if (d['_children'] != null) {
        d['_expanded'] = !d['_expanded'];
        globalSearchText();
        return;
      }

      if (isLoading) return true;
      isLoading = true;
      if (kDebugMode) print('expand ${d['_id']}');
      setLoading();
      var result = await EntityService.getData(entity,
          filter: filters,
          pageIndex: pageIndex,
          filterParentId: true,
          orderBy: orderBy,
          orderByDesc: orderByDesc,
          parentId: d['_id'].toString());

      setResume();
      isLoading = false;
      if (!result.success) {
        //data = [];
        return;
      }
      for (var r in result.data ?? []) {
        if (d['_tree_level'] != null)
          r['_tree_level'] = d['_tree_level'] + 1;
        else
          r['_tree_level'] = 1;
      }
      d['_children'] = result.data;
      d['_expanded'] = true;
      // data.insertAll(
      //     data.indexWhere((i) => i['_id'] == d['_id']) + 1, result.data);
      globalSearchText();
    } catch (err, t) {
      print('XXX expand item error');
      print(err);
      if (kDebugMode) print(t);
      setResume();
      isLoading = false;
    }
  }

  bool hasActions(Map<String, dynamic>? d) {
    if (entity.actions.length > 0) return true;
    if (entityActionsService == null) return false;
    return (entityActionsService?.getActions(d).data?.length ?? 0) > 0;
  }

  List<EAction> getActions(Map<String, dynamic>? d) {
    return EntityService.getActions(entity, d);
  }

  Future onAction(EAction action, Map<String, dynamic> data) async {
    try {
      await EntityService.onAction(entity, action, data, context);
      onRefresh();
    } catch (err) {
      showMessage('خطأ اثناء تنفيذ الاجراء', MessageType.error);
    }
  }

  lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }

  String get getPhotoDataPropName =>
      '${entity.properties.firstWhere((d) => d.isImage).dbName}Id';
}
