part of tecfy_corev2_client_package;

class MessageService {
  static bool connected = false;
  static IOWebSocketChannel? channel;
  static Stream<dynamic>? stream;
  static StreamController<Message>? messageStreamController;
  static Stream<Message>? messageStream;

  static init() async {
    if (ConfigService.token.isEmpty) return;
    messageStreamController = StreamController<Message>();
    messageStream = messageStreamController!.stream.asBroadcastStream();
  }

  static onMessageReceived(Message message) {
    saveLocal(message);
    messageStreamController?.add(message);
    // send service, confirm message recieved
    WebSocketService.sendPackage(
        Package(type: PackageType.messageACK, data: message.id));
    TimeoutMessage.show(message.fromUserId, message: message.body, onClick: () {
      AppNavigator.navigateTo(
          TecfyBasicApp.appGlobalkey.currentState!.context,
          () => ChatPage(User(
              id: message.fromUserId, recordNumber: 1, email: '????????')));
    }, timeoutMiniseconds: 5000);
  }

  static sendMessage(Message message) {
    saveLocal(message);
    WebSocketService.sendPackage(
        Package(type: PackageType.newMessage, data: message));
  }

  static Future<List<Message>?> getLocalMessages(String userId) async {
    var list = await DatabaseService.rawQuery(
        "select * from Message where (fromUserId = '$userId' and toUserId = '${LocalConfigService.user!.id}') or (toUserId = '$userId' and fromUserId = '${LocalConfigService.user!.id}') order by time desc");
    return list?.map((m) => Message.fromMapDatabase(m)).toList();
  }

  static Future<DateTime?> getLastMessageTime() async {
    var result = await DatabaseService.rawQuery(
        'select time from Message order by time desc limit 1');
    if (result?.length == 0) return null;
    return DateTime.fromMillisecondsSinceEpoch(
        (result?[0]['time'] as int?) ?? 0,
        isUtc: true);
  }

  static Future<bool> getNewMessages() async {
    try {
      var lastMessageTime = await getLastMessageTime();
      if (lastMessageTime == null)
        lastMessageTime = DateTime.now().toUtc().add(Duration(days: -300));
      var result = await TecfyBasicApp.apiService.httpGetDynamic(
          'Message/GetLatest/${intl.DateFormat("yyyy-MM-ddTHH:mm:ss").format(lastMessageTime.toLocal())}');
      if (result.success) {
        List<Message> mm =
            result.data.map<Message>((m) => Message.fromMapHistory(m)).toList();
        await saveLocalAll(mm);
        await markReceived(mm);
        return true;
      } else
        return false;
    } catch (err) {
      print('XXXXXXXXXX get new messages: $err');
      return false;
    }
  }

  static Future saveLocal(Message message) async {
    try {
      await DatabaseService.insert('Message', message.toMapDb());
    } catch (err) {
      print('XXXXXXXXXXXXXXXXXXXXXX cannot save message locally!!!!!!!');
      print(err);
    }
  }

  static Future saveLocalAll(List<Message> messages) async {
    await DatabaseService.exeBatch((batch) async {
      messages.forEach((m) => batch.insert('Message', m.toMapDb()));
      return OperationResult(success: true);
    }, true);
  }

  static Future<bool> markReceived(List<Message> messages) async {
    // TecfyBasicApp.apiService.httpGet('')
    return true;
  }
}
