part of tecfy_corev2_client_package;

class FileMultiProperty extends StatefulWidget {
  final List<String>? filesList;

  /// label string
  final String? label;
  final String? description;

  /// is required
  final bool isRequired;

  /// field id
  final String? fileId;

  /// field id
  final int maxSize;

  /// readOnly Filed
  final bool readOnly;

  // final int maxHeight;
  final double? aspectRatioMax;
  final double? aspectRatioMin;

  /// on changed
  final void Function(dynamic)? onChanged;

  FileMultiProperty(
      {this.filesList,
      this.label,
      this.description,
      this.maxSize = 200,
      // this.maxHeight = 200,
      this.aspectRatioMax,
      this.readOnly = false,
      this.aspectRatioMin,
      this.onChanged,
      this.fileId,
      this.isRequired = false}); //: assert(fileId != null );
  @override
  _FileMultiPropertyState createState() => _FileMultiPropertyState();
}

class _FileMultiPropertyState extends State<FileMultiProperty> {
  FilePickerResult? _selectedPhotos;
  ScrollController _scrollController = ScrollController();

  _onChoosePhoto() async {
    var maxWidth = widget.maxSize;
    // var maxHeight = widget.maxHeight;
    if (maxWidth == 0) maxWidth = 200;
    // if (maxHeight == 0) maxHeight = 200;
    _selectedPhotos = await FilePicker.platform.pickFiles(allowMultiple: true);
    if (_selectedPhotos != null) {
      AppNavigator.showLoading(context);
      List<String> photosUrlList = [];
      for (PlatformFile photo in _selectedPhotos!.files) {
        var result = await TecfyBasicApp.apiService.httpPostFile(
            'general/storage/file/fullUrl/admin', photo.bytes!, photo.name);
        widget.filesList!.add(result.data!);
        photosUrlList.add(result.data!);
      }
      widget.onChanged?.call(widget.filesList);
      print(photosUrlList);
      Navigator.pop(context);
      setState(() {});
    }
  }

  _onChoosePhotoFromFileManager() async {
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
      List<FileModel>? result = await AppNavigator.showAppDialogEx(context,
          () => FileManagerPage(isPopup: true, isMultipleChoices: true),
          size: DialogBoxSizes.large);
      if (result != null && result.isNotEmpty) {
        widget.filesList?.addAll(result.map((e) => e.url!));
      } else {
        return;
      }
      setState(() {
        widget.onChanged?.call(widget.filesList);
      });
    });
  }

  _onViewPhoto(String imageUrl) async {
    AppNavigator.showLoading(context);
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      AppNavigator.showAppDialogEx(
          context,
          () => PinchZoom(
              child: Image.network(getUrlEx(imageUrl)!), zoomEnabled: true),
          size: DialogBoxSizes.large);
    });
    AppNavigator.resume();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Tooltip(
      message: widget.description ?? '',
      child: widget.readOnly
          ? (widget.filesList != null &&
                  (widget.filesList?.isNotEmpty ?? false))
              ? Scrollbar(
                  controller: _scrollController,
                  interactive: true,
                  child: SingleChildScrollView(
                    scrollDirection: Axis.horizontal,
                    controller: _scrollController,
                    child: Row(
                        children: widget.filesList!
                            .map((e) => Container(
                                  margin: EdgeInsets.all(4),
                                  child: Stack(
                                    children: [
                                      InkWell(
                                        onDoubleTap: () async =>
                                            _onViewPhoto(e),
                                        child: Container(
                                          margin: EdgeInsets.only(
                                              left: 4, right: 4),
                                          constraints: BoxConstraints(
                                              maxWidth: 90, maxHeight: 120),
                                          decoration: BoxDecoration(
                                              border: Border.all(
                                                  color: Colors.grey, width: 2),
                                              image: DecorationImage(
                                                  image: NetworkImage(e),
                                                  fit: BoxFit.cover),
                                              borderRadius:
                                                  BorderRadius.circular(5)),
                                        ),
                                      ),
                                      Align(
                                        alignment: Alignment.topLeft,
                                        child: IconButton(
                                            tooltip:
                                                AppLocale.translate('Download'),
                                            onPressed: () async {
                                              if (await canLaunchUrl(
                                                  Uri.parse(getUrlEx(e)!))) {
                                                await launchUrl(
                                                    Uri.parse(getUrlEx(e)!));
                                              }
                                            },
                                            icon: Stack(
                                              children: [
                                                // this is icon outline to appeare in black images
                                                Icon(
                                                  Icons.download,
                                                  color: Colors.white,
                                                  size: 20,
                                                ),
                                                // this is icon body to appeare in white images
                                                Positioned.fill(
                                                  child: Icon(
                                                    Icons.download,
                                                    color: Colors.black,
                                                    size: 20,
                                                  ),
                                                ),
                                              ],
                                            )),
                                      ),
                                    ],
                                  ),
                                ))
                            .toList()),
                  ),
                )
              : const SizedBox.shrink()
          : Column(
              children: [
                DropdownButtonHideUnderline(
                  child: DropdownButton2(
                      items: [
                        DropdownMenuItem(
                            value: 'upload',
                            onTap: _onChoosePhoto,
                            child: Text('Upload')),
                        DropdownMenuItem(
                          value: 'upload',
                          onTap: _onChoosePhotoFromFileManager,
                          child: Text('Select from File Manager',
                              overflow: TextOverflow.ellipsis, maxLines: 1),
                        ),
                      ],
                      onChanged: (value) {},
                      customButton: Row(
                        children: [
                          Icon(Icons.photo_library_rounded),
                          Text('إضافة صور'),
                        ],
                      )),
                ),
                SizedBox(
                  height: 15,
                ),
                if (widget.filesList != null)
                  SingleChildScrollView(
                    scrollDirection: Axis.horizontal,
                    child: Row(
                        children: widget.filesList!
                            .map((e) => Padding(
                                  padding: EdgeInsets.only(left: 10),
                                  child: Container(
                                    width: 100,
                                    height: 100,
                                    decoration: BoxDecoration(
                                        border: Border.all(
                                            color: Colors.grey, width: 2),
                                        borderRadius: BorderRadius.circular(5)),
                                    child: Stack(
                                      alignment: Alignment.bottomLeft,
                                      children: <Widget>[
                                        Align(
                                          alignment: Alignment.center,
                                          child: Image.network(e,
                                              width: 100, fit: BoxFit.fitWidth),
                                        ),
                                        IconButton(
                                            onPressed: () {
                                              setState(() {
                                                widget.filesList?.remove(e);
                                                widget.onChanged
                                                    ?.call(widget.filesList);
                                              });
                                            },
                                            icon: Icon(
                                              Icons.delete_forever_rounded,
                                              color: Colors.red,
                                              size: 22,
                                            ))
                                      ],
                                    ),
                                  ),
                                ))
                            .toList()),
                  ),
              ],
            ),
    );
  }
}
