part of tecfy_corev2_client_package;

class PorpertyComponent extends StatefulWidget {
  /// the property object
  final Property property;

  /// the main entity
  final Entity? entity;

  /// change the value style
  final TextStyle? valueStyle;

  /// if we don't need to show label
  final bool noLabel;

  /// is add new so we can show or hide according to property config
  final bool isAddNew;

  /// on change
  final void Function(dynamic)? onChange;

  /// on submit
  final void Function(dynamic)? onSubmit;

  /// on update
  final void Function(Map<String, dynamic>)? onUpdateData;
  //final dynamic initalValue;
  /// if this property is a dropdown and it's filterd by another field
  ///final dynamic filterValue;
  final Map<String, dynamic> data;

  /// data that comes from the parent widget[rootEntity]
  final Map<String, dynamic>? rootData;

  /// the parent Entity
  final Entity? rootEntity;

  /// show all property as read only
  final bool readOnly;

  ///isFoucsed
  //final bool isFoucs;

  /// on open Popup
  final void Function()? onOpen;

  /// on close popup
  final void Function()? onClose;
  PorpertyComponent({
    required this.property,
    Key? key,
    this.entity,
    this.onChange,
    this.onSubmit,
    this.isAddNew = false,
    this.onUpdateData,
    this.valueStyle,
    this.noLabel = false,
    //this.filterValue,
    required this.data,
    this.rootData,
    this.rootEntity,
    this.readOnly = false,
    this.onOpen,
    this.onClose,
  }) : super(key: key); //: assert(entity != null || property.isRequired ==
  // false);
  @override
  _PorpertyComponentState createState() => _PorpertyComponentState();
}

class _PorpertyComponentState extends State<PorpertyComponent> {
  TextEditingController? _textController;
  FocusNode? _focusNode;
  dynamic _initValue;
  dynamic _dynamicProperties;
  String? _dynamicPropertiesId;

  //bool isVisable =true;
  bool _isVisible = true;
  @override
  void initState() {
    _init();
    super.initState();
  }

  bool get readOnly => widget.isAddNew
      ? widget.property.readOnly
      : widget.property.readOnlyOnEdit;

  void _init() {
    _initValue = MapService.getValue(widget.data, widget.property.dbName);
    var val = _initValue?.toString();
    if (widget.property.propertyType == PropertyTypes.Number && val != null) {
      if (val.contains('.'))
        val = (_initValue as num).toDouble().toStringAsFixed(2);
    }
    if (widget.property.propertyType == PropertyTypes.Text ||
        widget.property.propertyType == PropertyTypes.Number) {
      if (_textController == null) {
        _textController = TextEditingController(text: val ?? '');
        _focusNode = FocusNode();
        _textController!.addListener(() {
          _submitChanges();
        });
      } else {
        _textController?.text = val ?? '';
      }
    }
  }

  void _submitChanges() {
    if (widget.property.propertyType == PropertyTypes.Text) {
      _initValue = _textController?.text;
      if (widget.onChange != null) widget.onChange?.call(_textController?.text);
    }
    if (widget.property.propertyType == PropertyTypes.Number) {
      _initValue = num.tryParse(_textController?.text ?? '');
      if (widget.onChange != null) widget.onChange?.call(_initValue);
    }
  }

  // _getInitValueString() {
  //   return _initValue == null
  //       ? '__'
  //       : _initValue is int
  //           ? _initValue.toString()
  //           : _initValue is num
  //               ? (_initValue as num).toDouble().toStringAsFixed(2)
  //               : _initValue.toString();
  // }

  @override
  Widget build(BuildContext context) {
    if (widget.property.componentBuilder != null) {
      return widget.property.componentBuilder!
          .call(widget.data, widget.onUpdateData);
    }
    widget.property.isVisible(widget.data, widget.entity).then((isVisible) {
      if (isVisible != _isVisible) {
        _isVisible = isVisible;
        setState(() {});
      }
    });

    if (!_isVisible) return const SizedBox();

    //if (!widget.property.isVisable) return Container();
    if (_initValue != widget.data[widget.property.dbName]) _init();
    if (widget.readOnly) {
      if (widget.property.propertyType != PropertyTypes.File &&
          widget.property.propertyType != PropertyTypes.FileMulti) {
        return _getReadOnlyWidget();
      }
    }
    if (readOnly &&
        widget.property.propertyType != PropertyTypes.Dropdown &&
        widget.property.propertyType != PropertyTypes.OtherEntityMany &&
        widget.property.propertyType != PropertyTypes.ManyToMany &&
        widget.property.propertyType != PropertyTypes.OneToMany &&
        widget.property.propertyType != PropertyTypes.MultiDropdown &&
        widget.property.propertyType != PropertyTypes.File &&
        widget.property.propertyType != PropertyTypes.FileMulti) {
      if (widget.noLabel) {
        return SelectableText(widget.property.getDisplayText(widget.data));
      } else {
        return TextFormLabel(
          label: widget.property.name,
          value: widget.property.getDisplayText(widget.data),
        );
      }
    }

    switch (widget.property.propertyType) {
      case PropertyTypes.Dropdown:
        var prop = (widget.property as PropertyDropdown);
        if (prop.options != null) {
          return DropdownComponenet(
            key: UniqueKey(),
            rootData: widget.data,
            rootEntity: widget.entity,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            options: prop.options,
            readOnly: readOnly,
            displayProperty: LocalConfigService.isArabic ? 'nameAr' : 'nameEn',
            idProperty: '_id',
            initalValue:
                MapService.getValue(widget.data, prop.dbName)?.toString(),
            onChanged: (v) {
              // widget.data[prop.dbName] = v;
              MapService.updateValue(widget.data, prop.dbName, v);
              widget.onChange?.call(v);
              widget.onUpdateData?.call(widget.data);
              // widget.data[prop.dbName] = v;
              MapService.updateValue(widget.data, prop.dbName, v);
            },
          );
        }
        return DropdownAdvancedEntity(prop.entityName,
            rootData: widget.rootData,
            rootEntity: widget.rootEntity,
            onOpen: widget.onOpen,
            onClose: widget.onClose,
            isAddNew: widget.isAddNew,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            entity: EntityService.getEntity(prop.entityName)!,
            url: getUrlEx(prop.dropdownUrl),
            property: prop,
            readOnly: readOnly,
            data: widget.data, onChanged: (v) {
          widget.onChange?.call(v);
          if (widget.onUpdateData != null)
            widget.onUpdateData?.call(widget.data);
        });
      case PropertyTypes.MultiDropdown:
        var prop = (widget.property as PropertyMultiDropdown);
        return MultiDropdownComponent(
          key: UniqueKey(),
          entity: widget.entity!,
          description: widget.property.description,
          data: widget.data,
          url: getUrlEx(prop.dropdownUrl),
          urlFilterData: widget.data,
          property: prop,
          readOnly: readOnly,
          onChange: (List? list) {
            setState(() {});
          },
        );
      case PropertyTypes.OtherEntityMany:
        return ListFormProperty(
          widget.isAddNew,
          label: widget.noLabel ? null : widget.property.name,
          description: widget.property.description,
          isRequired: widget.noLabel ? false : widget.property.isRequired,
          readOnly: readOnly,
          entityId: (widget.property as PropertyOtherEntityMany).entityName,
          rootData: widget.data,
          rootEntity: widget.entity,
          data: _initValue == null
              ? []
              : _initValue is String
                  ? []
                  : _initValue,
          onChanged: (v) {
            widget.onChange?.call(v);
          },
        );

      case PropertyTypes.ManyToMany:
        if (MapService.getValue(widget.data, widget.property.dbName) == null) {
          MapService.updateValue(widget.data, widget.property.dbName, []);
        }
        if (!(widget.property as PropertyManyToMany).isWithData) {
          return ManyToManyProperty(
            label: widget.noLabel ? null : widget.property.name,
            // description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            selectedValues: _initValue,
            entity: EntityService.getEntity(
                (widget.property as PropertyManyToMany).entityName)!,
            onChanged: (v) => widget.onChange?.call(v),
          );
        }
        return ManyToManyWithData(
          (widget.property as PropertyManyToMany).entityName,
          label: widget.noLabel ? null : widget.property.name,
          // description: widget.property.description,
          data: widget.data,
          property: widget.property,
          isRequired: widget.noLabel ? false : widget.property.isRequired,
          items: MapService.getValue(widget.data, widget.property.dbName),
          entity: EntityService.getEntity(
              (widget.property as PropertyManyToMany).entityName)!,
          onChanged: (v) => widget.onChange?.call(v),
          entityMiddle:
              (widget.property as PropertyManyToMany).middleEntityName,
          // propertieMiddle:
          //     (widget.property as PropertyManyToMany).middleOtherPropertyName,
        );

      case PropertyTypes.OneToMany:
        return ManyToManyProperty(
          label: widget.noLabel ? null : widget.property.name,
          // description: widget.property.description,
          isRequired: widget.noLabel ? false : widget.property.isRequired,
          selectedValues: _initValue,
          entity: EntityService.getEntity(
              (widget.property as PropertyOneToMany).entityName)!,
          onChanged: (v) => widget.onChange?.call(v),
        );

      case PropertyTypes.Text:
        if ((widget.property as PropertyText).isBarcode) {
          return BarcodePropertyComponenet(
            labelText: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            textStyle: widget.valueStyle,
            focusNode: _focusNode,
            controller: _textController,
            onFieldSubmitted: (v) {
              _focusNode?.unfocus();
              widget.onSubmit?.call(_initValue);
            },
            onEditingComplete: _submitChanges,
          );
        }
        if ((widget.property as PropertyText).isMarkdown) {
          bool isShown = false;
          return Row(
            children: [
              IconButton(
                  icon: isShown
                      ? Icon(Icons.visibility)
                      : Icon(Icons.visibility_off),
                  onPressed: () {
                    isShown = !isShown;
                    setState(() {});
                  }),
              SizedBox(
                height: 250,
                width: 500,
                child: isShown
                    ? Markdown(
                        data: _initValue,
                        selectable: true,
                        onTapText: () {
                          isShown = !isShown;
                          setState(() {});
                        },
                      )
                    : MarkdownTextInput(
                        (String v) {
                          _initValue = v;
                          widget.onChange!.call(_initValue);
                          setState(() {});
                        },
                        _initValue.toString(),
                        controller: _textController,
                        actions: MarkdownType.values,
                      ),
              ),
            ],
          );
        }
        return AppTextFormField(
            textDirection: LocalConfigService.isArabic &&
                    !(widget.property as PropertyText).isEmail
                ? TextDirection.rtl
                : TextDirection.ltr,
            labelText: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            textStyle: widget.valueStyle,
            focusNode: _focusNode,
            controller: _textController,
            onFieldSubmitted: (v) {
              _focusNode?.unfocus();
              widget.onSubmit?.call(_initValue);
            },
            textInputAction: TextInputAction.next,
            obscureText: (widget.property as PropertyText).isPassword,
            keyboardType: (widget.property as PropertyText).isEmail
                ? TextInputType.emailAddress
                : (widget.property as PropertyText).lines > 1
                    ? TextInputType.multiline
                    : TextInputType.text,
            maxLines: (widget.property as PropertyText).lines,
            onEditingComplete: _submitChanges);
      case PropertyTypes.TextMultiLanguage:
        return MultiLangTextField(
            toggleBtnsDirection: Axis.vertical,
            isMarkdown: (widget.property as PropertyTextMultiLang).isMarkdown,
            data: MapService.getValue(widget.data, widget.property.dbName) ??
                <String, dynamic>{},
            textDirection: LocalConfigService.isArabic &&
                    !(widget.property as PropertyTextMultiLang).isEmail
                ? TextDirection.rtl
                : TextDirection.ltr,
            labelText: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            textStyle: widget.valueStyle,
            focusNode: _focusNode,
            onChangedEx: (Map<String, dynamic> s) => widget.onChange?.call(s),
            // controller: _textController,
            onFieldSubmitted: (v) {
              _focusNode?.unfocus();
              widget.onSubmit?.call(_initValue);
            },
            textInputAction: TextInputAction.next,
            obscureText: (widget.property as PropertyTextMultiLang).isPassword,
            keyboardType: (widget.property as PropertyTextMultiLang).isEmail
                ? TextInputType.emailAddress
                : (widget.property as PropertyTextMultiLang).lines > 1
                    ? TextInputType.multiline
                    : TextInputType.text,
            maxLines: (widget.property as PropertyTextMultiLang).lines,
            onEditingComplete: _submitChanges);
      case PropertyTypes.Html:
        return MultiLangHtmlEditor(
          labelText: widget.property.name,
          isRequired: widget.property.isRequired,
          description: widget.property.description,
          labelInHeader: true,
          data: MapService.getValue(widget.data, widget.property.dbName) ??
              <String, dynamic>{},
          onChangedEx: (Map<String, dynamic> s) => widget.onChange?.call(s),
        );
      case PropertyTypes.Number:
        return Directionality(
          textDirection: TextDirection.ltr,
          child: AppTextFormField(
              labelText: widget.noLabel ? null : widget.property.name,
              description: widget.property.description,
              isRequired: widget.noLabel ? false : widget.property.isRequired,
              controller: _textController,
              focusNode: _focusNode,
              onFieldSubmitted: (v) => _focusNode?.unfocus(),
              textStyle: widget.valueStyle,
              textInputAction: TextInputAction.next,
              keyboardType: TextInputType.number,
              onEditingComplete: _submitChanges),
        );
      case PropertyTypes.Boolean:
        return AppCheckBox(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            initalValue: _initValue,
            isFullrow: true,
            onChanged: (v) async {
              if (v == _initValue) return;
              widget.onChange?.call(v);
              if (widget.onUpdateData != null) {
                widget.onUpdateData?.call(widget.data);
              }
            });
      case PropertyTypes.File:
        Map<String, dynamic> map = {'_id': _initValue};
        PropertyFile property = widget.property as PropertyFile;
        return FileProperty(
            fileData: _initValue is Map ? _initValue : map,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            isImage: property.isImage,
            fileId: widget.data[widget.property.dbName + 'Id'],
            maxSize: (widget.property as PropertyFile).maxSize,
            // maxHeight: (widget.property as PropertyFile).maxHeight,
            aspectRatioMin: (widget.property as PropertyFile).aspectRatioMin,
            aspectRatioMax: (widget.property as PropertyFile).aspectRatioMax,
            fileExtensions: (widget.property as PropertyFile).fileExtensions,
            readOnly: property.isImage ? readOnly : widget.readOnly,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.Color:
        return ColorPropertyComponenet(
            initalValue: _initValue?.toString(),
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.MapArea:
        return MapAreaPropertyComponenet(
            initalValue: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.MarkImage:
        return MarkImagePropertyComponenet(
            initalValue: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.Date:
        return DateProperty(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.DateTime:
        return DateTimeProperty(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.Time:
        return TimeProperty(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.Location:
        return LocationPropertyComponent(
            key: UniqueKey(),
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initialValue: _initValue,
            isAddNew: widget.isAddNew,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.MapArea:
        return MapAreaPropertyComponenet(
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.FileMulti:
        return FileMultiProperty(
            filesList: _initValue == null ? [] : List<String>.from(_initValue),
            label: widget.noLabel ? null : widget.property.name,
            description: widget.property.description,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            fileId: widget.data[widget.property.dbName + 'Id'],
            maxSize: (widget.property as PropertyFile).maxSize,
            // maxHeight: (widget.property as PropertyFile).maxHeight,
            aspectRatioMin: (widget.property as PropertyFile).aspectRatioMin,
            aspectRatioMax: (widget.property as PropertyFile).aspectRatioMax,
            readOnly: widget.readOnly,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.DynamicProperties:
        //here i check if the flag of reading data from the root data exists and the root data isn't null then read data from the root data else read from prop data
        Map<String, dynamic> data =
            ((widget.property as PropertyDynamicProperties).isRootProp ==
                        true &&
                    widget.rootData != null &&
                    (widget.rootData?.isNotEmpty ?? false))
                ? widget.rootData ?? widget.data
                : widget.data;
        Entity? entity =
            ((widget.property as PropertyDynamicProperties).isRootProp ==
                        true &&
                    widget.rootEntity != null)
                ? widget.rootEntity ?? widget.entity
                : widget.entity;
        List? propList = _dynamicProperties ??
            MapService.getValue(data,
                (widget.property as PropertyDynamicProperties).propertyField);

        try {
          var propertyField =
              (widget.property as PropertyDynamicProperties).propertyField;
          var propName = propertyField.split('.').first;
          var id = MapService.getValue(data, '$propName._id');
          if ((propList == null || id != _dynamicPropertiesId)
              // && _dynamicProperties == null //commented to solve task #950 kafarat that dynamic properties not updated when updated data from propertyDropDown
              ) {
            var prop =
                entity?.properties.firstWhere((e) => e.dbName == propName);
            EntityService.getOne((prop as PropertyDropdown).entityName, id)
                .then((e) {
              if (e != null) {
                _dynamicProperties = MapService.getValue(
                    e,
                    ((widget.property as PropertyDynamicProperties)
                            .propertyField)
                        .substring(propName.length + 1));
                _dynamicPropertiesId = id;
                _dynamicProperties ??= [];
                propList = _dynamicProperties;
                setState(() {});
              }
            });
          }
        } catch (err) {}
        List<Property>? properties = propList
            ?.map((e) => Property.getProperty(e))
            .where((e) => e != null)
            .cast<Property>()
            .toList();
        if (properties == null || properties.isEmpty) return SizedBox();
        return InLineForm(
          null,
          true,
          data: widget.data,
          properties: properties,
        );
      default:
        return SelectableText(
            '${widget.property.dbName} ${widget.property.propertyType}');
    }
  }

  Widget _getReadOnlyWidget() {
    switch (widget.property.propertyType) {
      case PropertyTypes.Dropdown:
        var property = (widget.property as PropertyDropdown);
        var entity = EntityService.getEntity(property.entityName);
        String val;
        if (property.options != null) {
          val = widget.data[property.dbName] ?? '-';
        } else if (widget.data[property.dbName] == null) {
          EntityService.getOne(
                  entity!.dbName, widget.data[widget.property.dbName])
              .then((result) {
            if (result != null) {
              setState(() {
                widget.data[property.dbName] = result;
              });
            }
          });
          val = '__';
        } else {
          var valData =
              widget.data[property.dbName][entity!.displayField.dbName];
          if (valData is Map) {
            val = widget.data[property.dbName][entity.displayField.dbName]
                        [LocalConfigService.languagePrefix.toLowerCase()]
                    ?.toString() ??
                widget.data[property.dbName]['display']
                    [LocalConfigService.languagePrefix.toLowerCase()] ??
                widget.data[property.dbName]['name']
                    [LocalConfigService.languagePrefix.toLowerCase()] ??
                widget.data[property.dbName]['nameAr']
                    [LocalConfigService.languagePrefix.toLowerCase()] ??
                widget.data[property.dbName]['nameEn']
                    [LocalConfigService.languagePrefix.toLowerCase()] ??
                widget.data[property.dbName]['title']
                    [LocalConfigService.languagePrefix.toLowerCase()] ??
                '__';
          } else {
            val = widget.data[property.dbName][entity.displayField.dbName]
                    ?.toString() ??
                widget.data[property.dbName]['display'] ??
                widget.data[property.dbName]['name'] ??
                widget.data[property.dbName]['nameAr'] ??
                widget.data[property.dbName]['nameEn'] ??
                widget.data[property.dbName]['title'] ??
                '__';
          }
        }
        if (widget.noLabel)
          return SelectableText(val, style: widget.valueStyle);
        else
          return TextFormLabel(
            label: widget.property.name,
            description: widget.property.description,
            valueStyle: widget.valueStyle,
            value: val,
          );

      case PropertyTypes.OtherEntityMany:
        return ListFormProperty(widget.isAddNew,
            readOnly: true,
            description: widget.property.description,
            label: widget.noLabel ? null : widget.property.name,
            entityId: (widget.property as PropertyOtherEntityMany).entityName,
            data: _initValue ?? [],
            rootData: widget.data,
            onChanged: (v) {});
      default:
        if (widget.noLabel)
          return SelectableText(widget.property.getDisplayText(widget.data),
              style: widget.valueStyle);
        try {
          if (widget.property.propertyType == PropertyTypes.File &&
              (widget.property.isImage == false)) {
            return TextFormWidget(
              description: widget.property.description,
              label: widget.noLabel ? null : widget.property.name,
              value: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  widget.property.getDisplayWidget(widget.data,
                      textStyle: widget.valueStyle),
                  IconButton(
                      tooltip: AppLocale.translate('Download'),
                      onPressed: () async {
                        if (widget.data[widget.property.dbName] != null) {
                          if (widget.data[widget.property.dbName]!
                              .startsWith('https')) {
                            if (await canLaunchUrl(Uri.parse(
                                widget.data[widget.property.dbName]!))) {
                              await launchUrl(Uri.parse(
                                  widget.data[widget.property.dbName]!));
                            }
                          }
                        }
                      },
                      icon: Icon(Icons.download))
                ],
              ),
            );
          }
          return TextFormWidget(
            description: widget.property.description,
            label: widget.noLabel ? null : widget.property.name,
            value: widget.property
                .getDisplayWidget(widget.data, textStyle: widget.valueStyle),
          );
        } catch (err) {
          return TextFormLabel(
              description: widget.property.description,
              label: widget.property.name,
              value: widget.property.getDisplayText(widget.data),
              valueStyle: widget.valueStyle);
        }
    }
  }
}
